@@ -46,6 +46,7 @@ class PrinterError(Exception):
 
 class DriverError(Exception):
     "Base exception for all printer errors"
+
     def __init__(self, error='', code=-1):
         if code != -1:
             error = '%d: %s' % (code, error)
@@ -53,6 +54,10 @@ class DriverError(Exception):
         self.code = code
 
 
+class USBDriverError(DriverError):
+    """A USB Error that was raised by StoqDrivers (and not pyusb)"""
+
+
 class OutofPaperError(DriverError):
     "No paper left"
 
@@ -147,3 +152,7 @@ class AlreadyTotalized(DriverError):
 
 class InvalidValue(DriverError):
     "The value specified is invalid or is not in the expected range"
+
+
+class InvalidReplyException(DriverError):
+    "Invalid reply exception raised"
