@@ -1,31 +1,31 @@
 # -*- Mode: Python; coding: iso-8859-1 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 
-##
-## Stoqdrivers
-## Copyright (C) 2005-2007 Async Open Source <http://www.async.com.br>
-## All rights reserved
-##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-## USA.
-##
-## Author(s):   Cleber Rodrigues      <cleber@globalred.com.br>
-##              Henrique Romano       <henrique@async.com.br>
-##              Johan Dahlin          <jdahlin@async.com.br>
-##              Ronaldo Maia          <romaia@async.com.br>
-##
+#
+# Stoqdrivers
+# Copyright (C) 2005-2007 Async Open Source <http://www.async.com.br>
+# All rights reserved
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+# USA.
+#
+# Author(s):   Cleber Rodrigues      <cleber@globalred.com.br>
+#              Henrique Romano       <henrique@async.com.br>
+#              Johan Dahlin          <jdahlin@async.com.br>
+#              Ronaldo Maia          <romaia@async.com.br>
+#
 """
 Bematech MP25 driver
 """
@@ -35,11 +35,9 @@ from decimal import Decimal
 import logging
 import struct
 
-from kiwi.currency import currency
-from kiwi.python import Settable
-from zope.interface import implements
+from zope.interface import implementer
 
-from stoqdrivers.serialbase import SerialBase
+from stoqdrivers.enum import TaxType, UnitType
 from stoqdrivers.exceptions import (DriverError, OutofPaperError, PrinterError,
                                     CommandError, CouponOpenError,
                                     HardwareFailure,
@@ -47,10 +45,12 @@ from stoqdrivers.exceptions import (DriverError, OutofPaperError, PrinterError,
                                     ItemAdditionError, CancelItemError,
                                     CouponTotalizeError, CouponNotOpenError)
 from stoqdrivers.interfaces import ICouponPrinter
-from stoqdrivers.printers.capabilities import Capability
 from stoqdrivers.printers.base import BaseDriverConstants
-from stoqdrivers.enum import TaxType, UnitType
+from stoqdrivers.printers.capabilities import Capability
+from stoqdrivers.printers.fiscal import SintegraData
+from stoqdrivers.serialbase import SerialBase
 from stoqdrivers.translation import stoqdrivers_gettext
+from stoqdrivers.utils import bytes2str, str2bytes, encode_text
 
 _ = stoqdrivers_gettext
 
@@ -96,7 +96,7 @@ NAK = 21
 ACK = 6
 STX = 2
 
-RETRIES_BEFORE_TIMEOUT = 5
+RETRIES_BEFORE_TIMEOUT = 40
 CHARS_LIMIT = 492
 ALLOW_CANCEL_FISCAL_COUPON = 32
 
@@ -179,7 +179,7 @@ class MP25Status(object):
         # 2: (PrinterError(_("Owner data (CGC/IE) not programmed on the printer"))),
         # FIXME: This shouldn't be commented. But it will break the tests.
         # Need to update the tests for all bematech printers
-        #1: (CommandError(_("Command not executed")))
+        # 1: (CommandError(_("Command not executed")))
     }
 
     st3_codes = {
@@ -270,13 +270,23 @@ def dec2bin(n, trim=-1):
             a = ("0" * (trim - len(a))) + a
     return a
 
+
+def _rbytes2str(data):
+    if isinstance(data, tuple):
+        data = list(data)
+        return tuple(_rbytes2str(b) for b in data)
+    elif isinstance(data, bytes):
+        return bytes2str(data)
+    else:
+        return data
+
 #
 # Driver implementation
 #
 
 
+@implementer(ICouponPrinter)
 class MP25(SerialBase):
-    implements(ICouponPrinter)
     CMD_PROTO = 0x1c
 
     supported = True
@@ -292,8 +302,8 @@ class MP25(SerialBase):
 
     def __init__(self, port, consts=None):
         self._consts = consts or MP25Constants
-        port.setTimeout(2)
-        port.setWriteTimeout(5)
+        port.timeout = 20
+        port.writeTimeout = 20
         SerialBase.__init__(self, port)
         # XXX: Seems that Bematech doesn't contains any variable with the
         # coupon remainder value, so I need to manage it by myself.
@@ -322,11 +332,13 @@ class MP25(SerialBase):
         """
 
         command = chr(self.CMD_PROTO) + command
-        return struct.pack('<bH%dsH' % len(command),
-                           STX,
-                           len(command) + 2,
-                           command,
-                           sum([ord(i) for i in command]))
+        packet = struct.pack(
+            '<bH%dsH' % len(command),
+            STX,
+            len(command) + 2,
+            str2bytes(command),
+            sum([ord(i) for i in command]))
+        return bytes2str(packet)
 
     def _read_reply(self, size):
         a = 0
@@ -378,7 +390,7 @@ class MP25(SerialBase):
 
         format = self.reply_format % fmt
         reply = self._read_reply(struct.calcsize(format))
-        retval = struct.unpack(format, reply)
+        retval = _rbytes2str(struct.unpack(format, str2bytes(reply)))
 
         if raw:
             return retval
@@ -499,7 +511,7 @@ class MP25(SerialBase):
             if ord(c) == 0x03:
                 break
 
-            data += unicode(c, self.coupon_printer_charset)
+            data += encode_text(c, self.coupon_printer_charset)
 
         return data
 
@@ -587,7 +599,7 @@ class MP25(SerialBase):
         last_item = self._get_last_item_id()
         if item_id is None:
             item_id = last_item
-        elif item_id not in xrange(1, last_item + 2):
+        elif item_id not in range(1, last_item + 2):
             raise CancelItemError("There is no such item with ID %r"
                                   % item_id)
         self._send_command(CMD_CANCEL_ITEM, "%04d" % (item_id,))
@@ -602,7 +614,7 @@ class MP25(SerialBase):
             self.remainder_value = Decimal("0.0")
         return self.remainder_value
 
-    def coupon_totalize(self, discount=currency(0), markup=currency(0),
+    def coupon_totalize(self, discount=Decimal(0), markup=Decimal(0),
                         taxcode=TaxType.NONE):
 
         if discount:
@@ -685,9 +697,7 @@ class MP25(SerialBase):
         return self._consts
 
     def query_status(self):
-        #return '\x02\x05\x00\x1b#(f\x00'
-        query = self._create_packet(chr(CMD_READ_REGISTER) +
-                                    chr(self.registers.SERIAL))
+        query = self._create_packet(chr(CMD_READ_REGISTER) + chr(self.registers.SERIAL))
         return query
 
     def status_reply_complete(self, reply):
@@ -710,7 +720,7 @@ class MP25(SerialBase):
 
     def get_tax_constants(self):
         status = self._read_register(self.registers.TOTALIZERS)
-        status = struct.unpack('>H', status)[0]
+        status = struct.unpack('>H', str2bytes(status))[0]
 
         length, data = self._send_command(CMD_READ_TAXCODES, response='b32s')
 
@@ -765,7 +775,7 @@ class MP25(SerialBase):
 
         length, names = self._send_command(CMD_READ_TAXCODES, response='b32s')
         status = self._read_register(self.registers.TOTALIZERS)
-        status = struct.unpack('>H', status)[0]
+        status = struct.unpack('>H', str2bytes(status))[0]
         values = self._send_command(CMD_READ_TOTALIZERS, response='219s')
 
         taxes = []
@@ -786,7 +796,7 @@ class MP25(SerialBase):
         taxes.append(('F', bcd2dec(values[126:133]) / Decimal(100), 'ICMS'))
         date = bcd2hex(opening_date[:6])
 
-        return Settable(
+        return SintegraData(
             opening_date=datetime.date(year=2000 + int(date[4:6]),
                                        month=int(date[2:4]),
                                        day=int(date[:2])),
