@@ -1,46 +1,41 @@
 # -*- Mode: Python; coding: iso-8859-1 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 
-##
-## Stoqdrivers
-## Copyright (C) 2005-2007 Async Open Source <http://www.async.com.br>
-## All rights reserved
-##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-## USA.
-##
-## Author(s):   Johan Dahlin     <jdahlin@async.com.br>
-##              Henrique Romano  <henrique@async.com.br>
-##
+#
+# Stoqdrivers
+# Copyright (C) 2005-2007 Async Open Source <http://www.async.com.br>
+# All rights reserved
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+# USA.
+#
+# Author(s):   Johan Dahlin     <jdahlin@async.com.br>
+#              Henrique Romano  <henrique@async.com.br>
+#
 """
 Daruma FS345 driver
 """
+
 import datetime
 from decimal import Decimal
 import logging
 import time
 
-from kiwi.currency import currency
-from kiwi.python import Settable
-from zope.interface import implements
+from zope.interface import implementer
 
 from stoqdrivers import abicomp
-from stoqdrivers.serialbase import SerialBase
-from stoqdrivers.interfaces import ICouponPrinter
-from stoqdrivers.printers.capabilities import Capability
-from stoqdrivers.printers.base import BaseDriverConstants
 from stoqdrivers.enum import TaxType, UnitType
 from stoqdrivers.exceptions import (DriverError, PendingReduceZ,
                                     HardwareFailure, ReduceZError,
@@ -48,7 +43,13 @@ from stoqdrivers.exceptions import (DriverError, PendingReduceZ,
                                     OutofPaperError, PrinterOfflineError,
                                     CouponOpenError, CancelItemError,
                                     CloseCouponError)
+from stoqdrivers.interfaces import ICouponPrinter
+from stoqdrivers.printers.base import BaseDriverConstants
+from stoqdrivers.printers.capabilities import Capability
+from stoqdrivers.printers.fiscal import SintegraData
+from stoqdrivers.serialbase import SerialBase
 from stoqdrivers.translation import stoqdrivers_gettext
+from stoqdrivers.utils import encode_text, decode_text
 
 abicomp.register_codec()
 
@@ -148,11 +149,10 @@ class FS345Constants(BaseDriverConstants):
     }
 
 
+@implementer(ICouponPrinter)
 class FS345(SerialBase):
     log_domain = 'fs345'
 
-    implements(ICouponPrinter)
-
     supported = True
     model_name = "Daruma FS 345"
     coupon_printer_charset = "abicomp"
@@ -177,7 +177,7 @@ class FS345(SerialBase):
             if retval.startswith(':E'):
                 try:
                     self.handle_error(retval, raw)
-                except DriverError, e:
+                except DriverError as e:
                     if e.code == 42:
                         self.send_command(CMD_GET_X)
                         continue
@@ -220,11 +220,11 @@ class FS345(SerialBase):
             raise HardwareFailure('Broken status reply')
 
         if verbose:
-            print '== STATUS =='
+            print('== STATUS ==')
 
             # Codes found on page 57-59
-            print 'Raw status code:', status
-            print 'Cashier drawer is', ifset(status[1], 3, 'closed', 'open')
+            print('Raw status code:', status)
+            print('Cashier drawer is', ifset(status[1], 3, 'closed', 'open'))
 
         if self.has_pending_reduce():
             raise PendingReduceZ(_('Pending Reduce Z'))
@@ -236,26 +236,19 @@ class FS345(SerialBase):
             raise OutofPaperError(_('No paper'))
         if self.status_check(status, 2, 3):
             raise PrinterOfflineError(_("Offline"))
-        #if not self.status_check(status, 2, 2):
-        #     raise CommError(_("Peripheral is not connected to AUX"))
         if self.status_check(status, 2, 0):
             log.info('Almost out of paper')
 
         if verbose:
             S3 = status[3]
-            print ifset(S3, 3, 'Maintenance', 'Operational'), 'mode'
-            print 'Authentication', ifset(S3, 2, 'disabled', 'enabled')
-            print 'Guillotine', ifset(S3, 1, 'disabled', 'enabled')
-            print 'Auto close CF?', ifset(S3, 0, 'no', 'yes')
+            print(ifset(S3, 3, 'Maintenance', 'Operational'), 'mode')
+            print('Authentication', ifset(S3, 2, 'disabled', 'enabled'))
+            print('Guillotine', ifset(S3, 1, 'disabled', 'enabled'))
+            print('Auto close CF?', ifset(S3, 0, 'no', 'yes'))
 
         if self.status_check(status, 6, 1):
             raise ReduceZError(_("readZ is already emitted"))
 
-        # FIXME: I am not sure we should be doing this here. This method
-        # should only check the status, and not emit any other command.
-        #if self.needs_read_x(status):
-        #    self.send_command(CMD_GET_X)
-
         return status
 
     def has_pending_reduce(self, status=None):
@@ -310,31 +303,31 @@ class FS345(SerialBase):
         self._check_status(verbose=True)
 
     def show_information(self):
-        print 'Model:', self.send_command(CMD_GET_MODEL)
-        print 'Firmware:', self.send_command(CMD_GET_FIRMWARE)
+        print('Model:', self.send_command(CMD_GET_MODEL))
+        print('Firmware:', self.send_command(CMD_GET_FIRMWARE))
         data = self.send_command(CMD_LAST_RECORD)
 
         tt = time.strptime(data[:12], '%d%m%y%H%M%S')
-        print 'Last record:', time.strftime('%c', tt)
-        print 'Configuration:', self.send_command(CMD_GET_CONFIGURATION)
+        print('Last record:', time.strftime('%c', tt))
+        print('Configuration:', self.send_command(CMD_GET_CONFIGURATION))
 
     def show_document_status(self):
-        print '== DOCUMENT STATUS =='
+        print('== DOCUMENT STATUS ==')
         value = self._get_document_status()
-        print 'ECF:', value[2:6]
+        print('ECF:', value[2:6])
         document_type = value[6]
         if document_type == CLOSED_COUPON:
-            print 'No open coupon'
+            print('No open coupon')
         elif document_type == OPENED_FISCAL_COUPON:
-            print 'Document is a coupon (%s)' % value[7:12]
+            print('Document is a coupon (%s)' % value[7:12])
         else:
-            print 'Document type:', value[6]
+            print('Document type:', value[6])
 
         tt = time.strptime(value[13:27], '%H%M%S%d%m%Y')
-        print 'Current date/time:', time.strftime('%c', tt)
+        print('Current date/time:', time.strftime('%c', tt))
 
-        print 'Sum', int(value[27:41]) / 100.0
-        print 'GT atual', value[41:59]
+        print('Sum', int(value[27:41]) / 100.0)
+        print('GT atual', value[41:59])
 
     def _get_document_status(self):
         status = self.send_command(CMD_GET_DOCUMENT_STATUS)
@@ -484,7 +477,7 @@ class FS345(SerialBase):
         """Cancel the last non fiscal coupon or the last sale."""
         self.send_command(CMD_CANCEL_COUPON)
 
-    def coupon_totalize(self, discount=currency(0), surcharge=currency(0),
+    def coupon_totalize(self, discount=Decimal(0), surcharge=Decimal(0),
                         taxcode=TaxType.NONE):
         self._check_status()
         self._verify_coupon_open()
@@ -499,19 +492,17 @@ class FS345(SerialBase):
             mode = 1
         else:
             mode = 1
-            value = currency(0)
+            value = Decimal(0)
         # Page 33
         data = '%s%012d' % (mode, int(value * Decimal("1e2")))
         rv = self.send_command(CMD_TOTALIZE_COUPON, data)
-        return currency(rv) / Decimal("1e2")
+        return Decimal(rv) / Decimal("1e2")
 
     def coupon_close(self, message=''):
         self._check_status()
         self._verify_coupon_open()
 
-        if (self._customer_name or
-                self._customer_address or
-                self._customer_document):
+        if (self._customer_name or self._customer_address or self._customer_document):
             customer_name = self._customer_name or _("No client")
             customer_document = self._customer_document or _("No document")
             customer_address = self._customer_address or _("No address")
@@ -519,7 +510,9 @@ class FS345(SerialBase):
                               "%- 84s%- 84s%- 84s" % (customer_name,
                                                       customer_address,
                                                       customer_document))
-        message = message.encode(self.coupon_printer_charset)
+        # NOTE: Do not try to encode the message again. It was already
+        # encoded by printers.fiscal before. By doing that we would be
+        # encoding it twice.
         try:
             self.send_command(CMD_CLOSE_COUPON, message + '\xff')
         except DriverError:
@@ -555,7 +548,7 @@ class FS345(SerialBase):
             if const[2] == '\xff':
                 continue
 
-            const = const.decode(self.coupon_printer_charset)
+            const = decode_text(const, self.coupon_printer_charset)
             constants.append((char, const.strip()))
 
         return constants
@@ -620,7 +613,7 @@ class FS345(SerialBase):
                 break
             data += line
 
-        data = unicode(data, 'cp860')
+        data = encode_text(data, 'cp860')
         return (ret, data)
 
     def till_read_memory_by_reductions(self, start, end):
@@ -672,7 +665,7 @@ class FS345(SerialBase):
 
     def get_crz(self):
         registries = self._get_registers()
-        return int(registries[39:42])
+        return int(registries[38:42])
 
     def get_tax_constants(self):
         tax_codes = self.send_command(CMD_GET_TAX_CODES)[1:]
@@ -711,13 +704,10 @@ class FS345(SerialBase):
         method_letter = 'ABCDEFGHIJKLMNOP'
         for i in range(16):
             method = raw[i * 18:i * 18 + 18]
-            # XXX V = vinculavel. Why Only These????
-            #if method[0] == 'V':
-            #    methods.append((method_letter[i], method[1:].strip()))
             if method[2] == '\xff':
                 continue
 
-            name = method[1:].strip().decode(self.coupon_printer_charset)
+            name = decode_text(method[1:].strip(), self.coupon_printer_charset)
             methods.append((method_letter[i], name))
 
         return methods
@@ -758,18 +748,18 @@ class FS345(SerialBase):
         if dates[:6] == '000000':
             opening_date = datetime.date.today()
         else:
-            d, m, y = map(int, [dates[:2], dates[2:4], dates[4:6]])
+            d, m, y = list(map(int, [dates[:2], dates[2:4], dates[4:6]]))
             opening_date = datetime.date(2000 + y, m, d)
 
         identifier = self.send_command(CMD_GET_IDENTIFIER)
-        return Settable(
+        return SintegraData(
             opening_date=opening_date,
             serial=identifier[1:9],
             serial_id=int(identifier[13:17]),
             coupon_start=int(registries[:6]),
-            coupon_end=int(registries[7:12]),
-            cro=int(registries[35:38]),
-            crz=int(registries[39:42]),  # FIXME: this is being fetched before the actual
+            coupon_end=int(registries[6:12]),
+            cro=int(registries[34:38]),
+            crz=int(registries[38:42]),  # FIXME: this is being fetched before the actual
             # reduction, so it will offset by one.
             coo=int(registries[6:12]),
             period_total=period_total,
