@@ -1,28 +1,28 @@
 # -*- Mode: Python; coding: iso-8859-1 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 
-##
-## Stoqdrivers
-## Copyright (C) 2005 Async Open Source <http://www.async.com.br>
-## All rights reserved
-##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-## USA.
-##
-## Author(s):   Henrique Romano <henrique@async.com.br>
-##
+#
+# Stoqdrivers
+# Copyright (C) 2005 Async Open Source <http://www.async.com.br>
+# All rights reserved
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+# USA.
+#
+# Author(s):   Henrique Romano <henrique@async.com.br>
+#
 """
 Dataregis EP 275 drivers
 """
@@ -31,7 +31,7 @@ import datetime
 from decimal import Decimal
 import time
 
-from zope.interface import implements
+from zope.interface import implementer
 
 from stoqdrivers.serialbase import SerialBase
 from stoqdrivers.interfaces import (IChequePrinter,
@@ -258,10 +258,9 @@ class CouponItem:
 # The driver implementation
 #
 
+@implementer(ICouponPrinter, IChequePrinter)
 class EP375(SerialBase, BaseChequePrinter):
 
-    implements(ICouponPrinter, IChequePrinter)
-
     model_name = "Dataregis 375 EP"
     cheque_printer_charset = "ascii"
     coupon_printer_charset = "ascii"
@@ -425,10 +424,10 @@ class EP375(SerialBase, BaseChequePrinter):
         # Create a package for the command and its parameters. Package
         # format:
         #
-        #         +---+-----------+------+---------+--------+--------+
-        #NAME:    |STX|CMD_COUNTER|CMD_ID|PARAMS_SZ|[PARAMS]|CHECKSUM|
-        #         +---+-----------+------+---------+--------+--------+
-        #N_BYTES:   1       1         1        1        ?        1
+        #          +---+-----------+------+---------+--------+--------+
+        # NAME:    |STX|CMD_COUNTER|CMD_ID|PARAMS_SZ|[PARAMS]|CHECKSUM|
+        #          +---+-----------+------+---------+--------+--------+
+        # N_BYTES:   1       1         1        1        ?        1
         #
         # Where:
         #
@@ -550,8 +549,7 @@ class EP375(SerialBase, BaseChequePrinter):
 
     def coupon_totalize(self, discount=Decimal("0.0"),
                         surcharge=Decimal("0.0"), taxcode=TaxType.NONE):
-        # FIXME: API changed: discount/surcharge was percentage,
-        # now is currency.
+        # FIXME: API changed: discount/surcharge was percentage, now is Decimal.
 
         # The callsite must check if discount and charge are used together,
         # if so must raise an exception -- here we have a second check for
