@@ -1,44 +1,43 @@
 # -*- Mode: Python; coding: utf-8 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 
-##
-## Stoqdrivers
-## Copyright (C) 2009 Async Open Source <http://www.async.com.br>
-## All rights reserved
-##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-## USA.
-##
-## Author(s):   Henrique Romano <henrique@async.com.br>
-##              Johan Dahlin <henrique@async.com.br>
-##
+#
+# Stoqdrivers
+# Copyright (C) 2009 Async Open Source <http://www.async.com.br>
+# All rights reserved
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+# USA.
+#
+# Author(s):   Henrique Romano <henrique@async.com.br>
+#              Johan Dahlin <henrique@async.com.br>
+#
 """
 FiscNet base driver implementation.
 """
 
-import ConfigParser
+
+import configparser
 import datetime
 from decimal import Decimal
 import logging
 import re
 import os
 
-from kiwi.currency import currency
-from kiwi.python import Settable
 from serial import PARITY_EVEN
-from zope.interface import implements
+from zope.interface import implementer
 
 from stoqdrivers.enum import PaymentMethodType, TaxType, UnitType
 from stoqdrivers.exceptions import (DriverError, PendingReduceZ,
@@ -49,11 +48,13 @@ from stoqdrivers.exceptions import (DriverError, PendingReduceZ,
                                     InvalidState, PendingReadX,
                                     CloseCouponError, CouponNotOpenError)
 from stoqdrivers.interfaces import ICouponPrinter, IChequePrinter
+from stoqdrivers.printers.base import BaseDriverConstants
 from stoqdrivers.printers.capabilities import Capability
 from stoqdrivers.printers.cheque import BaseChequePrinter, BankConfiguration
-from stoqdrivers.printers.base import BaseDriverConstants
-from stoqdrivers.translation import stoqdrivers_gettext
+from stoqdrivers.printers.fiscal import SintegraData
 from stoqdrivers.serialbase import SerialBase
+from stoqdrivers.translation import stoqdrivers_gettext
+from stoqdrivers.utils import bytes2str, encode_text, decode_text
 
 _ = stoqdrivers_gettext
 
@@ -114,14 +115,15 @@ class FiscNetConstants(BaseDriverConstants):
         #         PaymentMethodType.GIFT_CERTIFICATE: '5,
     }
 
+
 _RETVAL_TOKEN_RE = re.compile(r"^\s*([^=\s;]+)")
 _RETVAL_QUOTED_VALUE_RE = re.compile(r"^\s*=\s*\"([^\"\\]*(?:\\.[^\"\\]*)*)\"")
 _RETVAL_VALUE_RE = re.compile(r"^\s*=\s*([^\s;]*)")
 _RETVAL_ESCAPE_RE = re.compile(r"\\(.)")
 
 
+@implementer(IChequePrinter, ICouponPrinter)
 class FiscNetECF(SerialBase):
-    implements(IChequePrinter, ICouponPrinter)
 
     log_domain = 'FiscNet'
 
@@ -162,19 +164,19 @@ class FiscNetECF(SerialBase):
     }
 
     def __init__(self, port, consts=None):
-        port.setParity(PARITY_EVEN)
-        port.setWriteTimeout(3)
+        port.parity = PARITY_EVEN
+        port.writeTimeout = 3
 
         SerialBase.__init__(self, port)
         self._consts = consts or FiscNetConstants
         self._command_id = 0
-        config = ConfigParser.RawConfigParser()
+        config = configparser.RawConfigParser()
         has_conf = config.read([os.path.expanduser('~/.stoq/stoqdrivers.conf')])
         if has_conf:
             try:
                 self.CASH_SUPPLY = config.get('fiscnet', 'cash_supply')
                 self.CASH_REMOVAL = config.get('fiscnet', 'cash_removal')
-            except ConfigParser.NoOptionError:
+            except configparser.NoOptionError:
                 log.info("Option not found, check stoqdrivers.conf")
         else:
             log.info("file: stoqdrivers.conf not found")
@@ -224,10 +226,12 @@ class FiscNetECF(SerialBase):
     def _send_command(self, command, **params):
         # Page 38-39
         parameters = []
-        for param, value in params.items():
+        for param, value in sorted(params.items()):
             if isinstance(value, Decimal):
                 value = ('%.03f' % value).replace('.', ',')
-            elif isinstance(value, basestring):
+            elif isinstance(value, bytes):
+                value = '"%s"' % bytes2str(value)
+            elif isinstance(value, str):
                 value = '"%s"' % value
             elif isinstance(value, bool):
                 if value is False:
@@ -304,7 +308,7 @@ class FiscNetECF(SerialBase):
                 return datetime.date.today()
             else:
                 # "29/03/2007" -> datetime.date(2007, 3, 29)
-                d, m, y = map(int, retval[1:-1].split('/'))
+                d, m, y = list(map(int, retval[1:-1].split('/')))
                 return datetime.date(y, m, d)
         elif regtype == str:
             # '"string"' -> 'string'
@@ -335,7 +339,7 @@ class FiscNetECF(SerialBase):
         try:
             retdict = self._send_command(
                 'LeNaoFiscal', CodNaoFiscal=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8057:  # Not configured
                 raise
         else:
@@ -351,7 +355,7 @@ class FiscNetECF(SerialBase):
             self._send_command(
                 'DefineNaoFiscal', CodNaoFiscal=code, DescricaoNaoFiscal=name,
                 NomeNaoFiscal=name, TipoNaoFiscal=entrada)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8036:
                 raise
 
@@ -359,7 +363,7 @@ class FiscNetECF(SerialBase):
         try:
             self._send_command(
                 'ExcluiNaoFiscal', CodNaoFiscal=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8057:  # Not configured
                 raise
 
@@ -367,7 +371,7 @@ class FiscNetECF(SerialBase):
         try:
             retdict = self._send_command(
                 'LeMeioPagamento', CodMeioPagamentoProgram=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8014:  # Not configured
                 raise
         else:
@@ -380,19 +384,16 @@ class FiscNetECF(SerialBase):
             if not configure:
                 return
 
-        try:
-            self._send_command(
-                'DefineMeioPagamento',
-                CodMeioPagamentoProgram=code, DescricaoMeioPagamento=name,
-                NomeMeioPagamento=name, PermiteVinculado=vinculated)
-        except DriverError, e:
-            raise
+        self._send_command(
+            'DefineMeioPagamento',
+            CodMeioPagamentoProgram=code, DescricaoMeioPagamento=name,
+            NomeMeioPagamento=name, PermiteVinculado=vinculated)
 
     def _delete_payment_method(self, code):
         try:
             self._send_command(
                 'ExcluiMeioPagamento', CodMeioPagamentoProgram=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8014:  # Not configured
                 raise
 
@@ -400,7 +401,7 @@ class FiscNetECF(SerialBase):
         try:
             retdict = self._send_command(
                 'LeAliquota', CodAliquotaProgramavel=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8005:  # Not configured
                 raise
         else:
@@ -413,21 +414,18 @@ class FiscNetECF(SerialBase):
             if not configure:
                 return
 
-        try:
-            self._send_command(
-                'DefineAliquota',
-                CodAliquotaProgramavel=code,
-                DescricaoAliquota='%2.2f%%' % value,
-                PercentualAliquota=value,
-                AliquotaICMS=not service)
-        except DriverError, e:
-            raise
+        self._send_command(
+            'DefineAliquota',
+            CodAliquotaProgramavel=code,
+            DescricaoAliquota='%2.2f%%' % value,
+            PercentualAliquota=value,
+            AliquotaICMS=not service)
 
     def _delete_tax_code(self, code):
         try:
             self._send_command(
                 'ExcluiAliquota', CodAliquotaProgramavel=code)
-        except DriverError, e:
+        except DriverError as e:
             if e.code != 8005:  # Not configured
                 raise
 
@@ -465,27 +463,19 @@ class FiscNetECF(SerialBase):
         return taxes
 
     def _configure_printer(self):
-        self._define_tax_name(0,
-                              self.CASH_SUPPLY.encode(self.coupon_printer_charset), entrada=True)
-        self._define_tax_name(1,
-                              self.CASH_REMOVAL.encode(self.coupon_printer_charset), entrada=False)
+        enc = lambda text: encode_text(text, self.coupon_printer_charset)
+
+        self._define_tax_name(0, enc(self.CASH_SUPPLY), entrada=True)
+        self._define_tax_name(1, enc(self.CASH_REMOVAL), entrada=False)
         for code in range(2, 15):
             self._delete_tax_name(code)
 
-        self._define_payment_method(0,
-                                    u'Cheque'.encode(self.coupon_printer_charset))
-        self._define_payment_method(1,
-                                    u'Boleto'.encode(self.coupon_printer_charset))
-        self._define_payment_method(2,
-                                    u'Cartão credito'.encode(self.coupon_printer_charset),
-                                    vinculated=True)
-        self._define_payment_method(3,
-                                    u'Cartão debito'.encode(self.coupon_printer_charset),
-                                    vinculated=True)
-        self._define_payment_method(4,
-                                    u'Financeira'.encode(self.coupon_printer_charset))
-        self._define_payment_method(5,
-                                    u'Vale compra'.encode(self.coupon_printer_charset))
+        self._define_payment_method(0, enc('Cheque'))
+        self._define_payment_method(1, enc('Boleto'))
+        self._define_payment_method(2, enc('Cartão credito'), vinculated=True)
+        self._define_payment_method(3, enc('Cartão debito'), vinculated=True)
+        self._define_payment_method(4, enc('Financeira'))
+        self._define_payment_method(5, enc('Vale compra'))
         for code in range(6, 15):
             self._delete_payment_method(code)
 
@@ -500,17 +490,17 @@ class FiscNetECF(SerialBase):
 
     def print_status(self):
         status = self._get_status()
-        print 'Flags'
+        print('Flags')
         for flag in reversed(_status_flags):
             if status & flag:
-                print flag, _flagnames[flag]
+                print(flag, _flagnames[flag])
 
-        print 'non-fiscal registers'
+        print('non-fiscal registers')
         for i in range(15):
             try:
-                print self._send_command(
-                    'LeNaoFiscal', CodNaoFiscal=i)
-            except DriverError, e:
+                print(self._send_command(
+                    'LeNaoFiscal', CodNaoFiscal=i))
+            except DriverError as e:
                 if e.code != 8057:
                     raise
 
@@ -584,8 +574,8 @@ class FiscNetECF(SerialBase):
         """Cancel the last non fiscal coupon or the last sale."""
         self._send_command('CancelaCupom')
 
-    def coupon_totalize(self, discount=currency(0),
-                        surcharge=currency(0), taxcode=TaxType.NONE):
+    def coupon_totalize(self, discount=Decimal(0),
+                        surcharge=Decimal(0), taxcode=TaxType.NONE):
         # The FISCnet protocol (the protocol used in this printer model)
         # doesn't have a command to totalize the coupon, so we just get
         # the discount/surcharge values and applied to the coupon.
@@ -655,7 +645,7 @@ class FiscNetECF(SerialBase):
                                LeituraSimplificada=True,
                                DataInicial=start,
                                DataFinal=end)
-        except DriverError, e:
+        except DriverError as e:
             if e.code == 8089:
                 return
 
@@ -697,7 +687,7 @@ class FiscNetECF(SerialBase):
             try:
                 retdict = self._send_command(
                     'LeAliquota', CodAliquotaProgramavel=reg)
-            except DriverError, e:
+            except DriverError as e:
                 if e.code == 8005:  # Aliquota nao carregada
                     continue
                 raise
@@ -730,14 +720,15 @@ class FiscNetECF(SerialBase):
             try:
                 retdict = self._send_command(
                     'LeMeioPagamento', CodMeioPagamentoProgram=reg)
-            except DriverError, e:
+            except DriverError as e:
                 if e.code == 8014:  # Meio de pagamento nao carregado
                     continue
                 raise
 
             code = retdict['CodMeioPagamentoProgram']
             name = retdict['NomeMeioPagamento']
-            constants.append((code, name.decode(self.coupon_printer_charset)))
+            name = decode_text(name, self.coupon_printer_charset)
+            constants.append((code, name))
 
         return constants
 
@@ -760,7 +751,7 @@ class FiscNetECF(SerialBase):
         return self._read_register('VersaoSW', str)
 
     def get_sintegra(self):
-        data = Settable(
+        data = SintegraData(
             opening_date=self._read_register('DataAbertura', datetime.date),
             serial=self.get_serial(),
             serial_id=self._read_register('ECF', int),
@@ -784,7 +775,7 @@ class FiscNetECF(SerialBase):
                            COO=coo, Valor=value)
 
     def payment_receipt_print(self, text):
-        text = text.encode(self.coupon_printer_charset)
+        text = encode_text(text, self.coupon_printer_charset)
         for line in text.split('\n'):
             line = line.replace('\\', '\\\\')  # Vespague sucks
             self._send_command('ImprimeTexto', TextoLivre=line)
@@ -799,7 +790,7 @@ class FiscNetECF(SerialBase):
         self._send_command('AbreGerencial', CodGerencial=gerencial_id)
 
     def gerencial_report_print(self, text):
-        text = text.encode(self.coupon_printer_charset)
+        text = encode_text(text, self.coupon_printer_charset)
         for line in text.split('\n'):
             line = line.replace('\\', '\\\\')  # Vespague sucks
             self._send_command('ImprimeTexto', TextoLivre=line)
@@ -808,8 +799,8 @@ class FiscNetECF(SerialBase):
         self._send_command('EncerraDocumento')
 
 
+@implementer(IChequePrinter)
 class FiscNetChequePrinter(FiscNetECF, BaseChequePrinter):
-    implements(IChequePrinter)
 
     def __init__(self, port, consts=None):
         FiscNetECF.__init__(self, port, consts)
