@@ -1,54 +1,63 @@
 # -*- Mode: Python; coding: utf-8 -*-
 # vi:si:et:sw=4:sts=4:ts=4
 
-##
-## Stoqdrivers
-## Copyright (C) 2005-2011 Async Open Source <http://www.async.com.br>
-## All rights reserved
-##
-## This program is free software; you can redistribute it and/or modify
-## it under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 2 of the License, or
-## (at your option) any later version.
-##
-## This program is distributed in the hope that it will be useful,
-## but WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-## GNU General Public License for more details.
-##
-## You should have received a copy of the GNU General Public License
-## along with this program; if not, write to the Free Software
-## Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-## USA.
-##
-## Author(s):   Stoq Team  <stoq-devel@async.com.br>
-##
-##
+#
+# Stoqdrivers
+# Copyright (C) 2005-2011 Async Open Source <http://www.async.com.br>
+# All rights reserved
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
+# USA.
+#
+# Author(s):   Stoq Team  <stoq-devel@async.com.br>
+#
 """
 A simple implementation of a virtual printer.
 """
+
 import datetime
 from decimal import Decimal
 import json
 import os
+from unittest import mock
 
-import gtk
-import pango
-from kiwi.python import Settable
-from zope.interface import implements
+try:
+    from gi.repository import Gtk, Pango
+except ImportError:
+    Gtk = mock.Mock()
+    Pango = mock.Mock()
+from zope.interface import implementer
 
 from stoqdrivers.enum import PaymentMethodType, TaxType, UnitType
 from stoqdrivers.exceptions import (CouponTotalizeError, PaymentAdditionError,
                                     CloseCouponError, CouponOpenError,
                                     CancelItemError, ItemAdditionError,
                                     DriverError, PrinterOfflineError)
-from stoqdrivers.interfaces import ICouponPrinter
+from stoqdrivers.interfaces import ICouponPrinter, INonFiscalPrinter
 from stoqdrivers.printers.base import BaseDriverConstants
 from stoqdrivers.printers.capabilities import Capability
+from stoqdrivers.printers.fiscal import SintegraData
 from stoqdrivers.translation import stoqdrivers_gettext
 
 _ = stoqdrivers_gettext
 
+# Global state of the printer. Even if we create multiple instances of the printer during runtime
+# (which should not happen), all of them will share the same state.
+_printer_off = False
+_drawer_open = False
+
 
 class CouponItem:
     def __init__(self, id, quantity, value):
@@ -97,40 +106,62 @@ class FakeConstants(BaseDriverConstants):
     ]
 
 
-class OutputWindow(gtk.Window):
-    columns = 60
+class OutputWindow(Gtk.Window):
+    _instance = None
 
-    def __init__(self, printer):
-        self._printer = printer
-        gtk.Window.__init__(self)
+    def __init__(self):
+        self._printer = None
+        Gtk.Window.__init__(self)
         self.set_title(_("ECF Emulator"))
-        self.set_size_request(220, 320)
+        self.set_size_request(380, 320)
         self.move(0, 0)
         self.set_deletable(False)
-        self.vbox = gtk.VBox(0, False)
+        self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.add(self.vbox)
 
         self._create_ui()
 
-    def _create_ui(self):
-        sw = gtk.ScrolledWindow()
-        self.vbox.pack_start(sw, True, True)
-        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+    @classmethod
+    def get_instance(cls):
+        if cls._instance:
+            return cls._instance
+        cls._instance = cls()
+        return cls._instance
 
-        self.textview = gtk.TextView()
-        self.textview.modify_font(pango.FontDescription("Monospace 7"))
+    def set_printer(self, printer):
+        self._printer = printer
+
+    def _create_ui(self):
+        sw = Gtk.ScrolledWindow()
+        # FIXME: The overlay scrolling when a TextView is inside a
+        # ScrolledWindow is somewhat broken in a way that it would make it get
+        # a height of 0 when being displayed.
+        sw.set_property('overlay_scrolling', False)
+        self.vbox.pack_start(sw, True, True, 0)
+        sw.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.ALWAYS)
+
+        self.textview = Gtk.TextView()
+        self.textview.modify_font(Pango.FontDescription("Monospace 7"))
         sw.add(self.textview)
         self.buffer = self.textview.get_buffer()
 
-        buttonbox = gtk.HBox()
-        self.vbox.pack_start(buttonbox, False, False)
+        buttonbox = Gtk.HBox()
+        self.vbox.pack_start(buttonbox, False, False, 3)
 
-        self.b = gtk.ToggleButton(_("Turn off"))
+        self.b = Gtk.ToggleButton.new_with_label(_("Turn off"))
         self.b.set_active(True)
-        buttonbox.pack_start(self.b)
+        buttonbox.pack_start(self.b, True, True, 3)
         self.b.connect("toggled", self._on_onoff__toggled)
 
+        self.drawer = Gtk.ToggleButton.new_with_label(_("Open drawer"))
+        self.drawer.set_active(True)
+        buttonbox.pack_start(self.drawer, True, True, 3)
+        self.drawer.connect("toggled", self._on_open_drawer__toggled)
+
+        self.show_all()
+
     def _on_onoff__toggled(self, button):
+        assert self._printer
         if button.get_active():
             self.b.set_label(_("Turn off"))
             self._printer.set_off(False)
@@ -138,39 +169,39 @@ class OutputWindow(gtk.Window):
             self.b.set_label(_("Turn on"))
             self._printer.set_off(True)
 
+    def _on_open_drawer__toggled(self, button):
+        assert self._printer
+        if button.get_active():
+            self.drawer.set_label(_("Close drawer"))
+            self._printer.open_drawer()
+        else:
+            self.drawer.set_label(_("Open drawer"))
+            self._printer.close_drawer()
+
     def feed(self, text):
-        self.buffer.props.text += text
-        mark = self.buffer.get_insert()
-        self.textview.scroll_mark_onscreen(mark)
+        if isinstance(text, bytes):
+            text = text.decode()
+        position = self.buffer.get_end_iter()
+        self.buffer.insert(position, text)
 
-    def feed_line(self, text=None):
-        if not text:
-            self.buffer.props.text += ('-' * self.columns) + '\n'
-            return
-
-        length = (self.columns - len(text) - 2) / 2
-        self.buffer.props.text += '%s %s %s\n' % (
-            '-' * length, text, '-' * length)
         mark = self.buffer.get_insert()
         self.textview.scroll_mark_onscreen(mark)
 
 
+@implementer(ICouponPrinter, INonFiscalPrinter)
 class Simple(object):
-    implements(ICouponPrinter)
 
     model_name = "Virtual Printer"
     coupon_printer_charset = "utf-8"
 
-    identify_customer_at_end = True
+    identify_customer_at_end = False
     supported = False
+    max_characters = 72
 
     def __init__(self, port, consts=None):
         self._consts = consts or FakeConstants()
         self._customer_document = None
 
-        self._off = False
-        self.output = OutputWindow(self)
-
         # Internal state
         self.till_closed = False
         self.opening_date = datetime.date.today()
@@ -190,20 +221,21 @@ class Simple(object):
         self._reset_flags()
         self._load_state()
 
-        self.output.feed(
+    #
+    # Helper methods
+    #
+
+    def _print_coupon_header(self):
+        self.write(
             "Virtual Printer\n"
             "Test Company\n"
             "CNPJ: 00.000.000/0000-00 IE: ISENTO\n"
             "IM: 012345\n")
-        self.output.feed_line()
-        self.output.feed("%s  COO:%s\n" % (
+        self._feed_line()
+        self.write("%s  COO:%s\n" % (
             datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S"),
             self.coo))
-        self.output.feed_line()
-
-    #
-    # Helper methods
-    #
+        self._feed_line()
 
     def _get_state_filename(self):
         dirname = os.path.join(os.environ['HOME'], '.stoq')
@@ -242,10 +274,11 @@ class Simple(object):
         fp.close()
 
     def set_off(self, off):
-        self._off = off
+        global _printer_off
+        _printer_off = off
 
     def _check(self):
-        if self._off:
+        if _printer_off:
             raise PrinterOfflineError
 
     def _reset_flags(self):
@@ -256,6 +289,7 @@ class Simple(object):
         self.has_payments = False
         self.payments_total = Decimal("0.0")
         self._items = {}
+        self._is_centralized = False
 
     def _check_coupon_is_opened(self):
         if not self.is_coupon_opened:
@@ -265,6 +299,31 @@ class Simple(object):
         if self.is_coupon_opened:
             raise CouponOpenError(_("There is a coupon already open"))
 
+    def _feed_line(self):
+        self.write(('-' * self.max_characters) + '\n')
+
+    def open_drawer(self):
+        global _drawer_open
+        _drawer_open = True
+        OutputWindow.get_instance().drawer.set_label(_("Close drawer"))
+
+    def close_drawer(self):
+        global _drawer_open
+        _drawer_open = False
+        OutputWindow.get_instance().drawer.set_label(_("Open drawer"))
+
+    def is_drawer_open(self):
+        return _drawer_open
+
+    #
+    #   SerialBase implementation
+    #
+
+    def write(self, data):
+        output = OutputWindow.get_instance()
+        output.set_printer(self)
+        output.feed(data)
+
     #
     # ICouponPrinter implementation
     #
@@ -277,13 +336,6 @@ class Simple(object):
 
     def coupon_identify_customer(self, customer, address, document):
         self._check()
-        if document not in ['160.618.061-40',
-                            '335.728.854-00',
-                            '804.727.615-87',
-                            '871.007.004-42']:
-            raise ItemAdditionError(
-                _("Not allowed to sell to a client not created by the demo"))
-
         self._customer_name = customer
         self._customer_document = document
         self._customer_address = address
@@ -293,10 +345,10 @@ class Simple(object):
 
     def coupon_open(self):
         self._check()
-        self.output.feed('\n')
-        self.output.feed("CUPOM SIMULADO\n")
+        self.write('\n')
+        self.write("CUPOM SIMULADO\n")
 
-        self.output.feed("ITEM CODIGO DESCRICAO QTD.UN.VL. UNIT R$ ST A/T VL ITEM R$\n")
+        self.write("ITEM CODIGO DESCRICAO QTD.UN.VL. UNIT R$ ST A/T VL ITEM R$\n")
         self._check_coupon_is_closed()
         self.is_coupon_opened = True
 
@@ -305,12 +357,6 @@ class Simple(object):
                         discount=Decimal("0.0"),
                         surcharge=Decimal("0.0"), unit_desc=""):
         self._check()
-        if code not in ["{:02d}".format(x) for x in range(1, 11)]:
-            # Allow deliveries as well
-            if code != '' and taxcode != 'S0':
-                raise ItemAdditionError(
-                    _("Not allowed to sell an item not created by the demo"))
-
         self._check_coupon_is_opened()
         if self.is_coupon_totalized:
             raise ItemAdditionError(_("The coupon is already totalized, "
@@ -319,36 +365,32 @@ class Simple(object):
         item_id = self.items_quantity
         item = CouponItem(item_id, quantity, price)
         self._items[item_id] = item
-        self.output.feed("%03d %s %s\n" % (self.items_quantity, code, description))
-        self.output.feed("  %d %f %s\n" % (quantity, price, taxcode))
+        self.write("%03d %s %s\n" % (self.items_quantity, code, description))
+        self.write("  %d %f %s\n" % (quantity, price, taxcode))
         return item_id
 
     def coupon_cancel_item(self, item_id):
         self._check()
         self._check_coupon_is_opened()
-        if not item_id in self._items:
+        if item_id not in self._items:
             raise CancelItemError(_("There is no item with this ID (%d)")
                                   % item_id)
         elif self.is_coupon_totalized:
             raise CancelItemError(_("The coupon is already totalized, "
                                     "you can't cancel items anymore."))
         self._items.pop(item_id)
-        self.output.feed('cancel_item %r\n' % (item_id, ))
+        self.write('cancel_item %r\n' % (item_id, ))
 
     def coupon_cancel(self):
         self._check()
-        # FIXME: If we don't have a coupon open, verify that
-        #        we've opened at least one
-        #self._check_coupon_is_opened()
-        self.output.feed_line()
-        self.output.feed('    Cupom Cancelado\n')
-        self.output.feed_line()
+        self._feed_line()
+        self.write('    Cupom Cancelado\n')
+        self._feed_line()
         self._reset_flags()
 
     def coupon_totalize(self, discount=Decimal("0.0"),
                         surcharge=Decimal("0.0"), taxcode=TaxType.NONE):
-        # FIXME: API changed: discount/surcharge was percentage,
-        # now is currency.
+        # FIXME: API changed: discount/surcharge was percentage, now is Decimal.
         self._check()
         self._check_coupon_is_opened()
         if not self.items_quantity:
@@ -371,8 +413,8 @@ class Simple(object):
                                         "than zero!"))
 
         self.is_coupon_totalized = True
-        self.output.feed('\n')
-        self.output.feed('Pagamentos:\n')
+        self.write('\n')
+        self.write('Pagamentos:\n')
         return self.totalized_value
 
     def coupon_add_payment(self, payment_method, value, description=u"",
@@ -383,7 +425,7 @@ class Simple(object):
                                          "coupon since it isn't totalized"))
         self.payments_total += value
         self.has_payments = True
-        self.output.feed('  %s - %s\n' % (
+        self.write('  %s - %s\n' % (
             self._consts._payment_descriptions[payment_method], value))
         return self.totalized_value - self.payments_total
 
@@ -401,11 +443,11 @@ class Simple(object):
                                      "match the totalized value."))
         troco = self.payments_total - self.totalized_value
         if troco:
-            self.output.feed('Troco: %0.2f\n' % troco)
-        self.output.feed_line()
+            self.write('Troco: %0.2f\n' % troco)
+        self._feed_line()
         if message:
-            self.output.feed(message)
-        self.output.feed('\n')
+            self.write(message)
+        self.write('\n')
         self._reset_flags()
         return 0
 
@@ -433,8 +475,8 @@ class Simple(object):
 
     def summarize(self):
         self._check()
-        self.output.feed('LEITURA X\n')
-        self.output.feed_line()
+        self.write('LEITURA X\n')
+        self._feed_line()
         self.till_closed = False
         self._save_state()
 
@@ -448,27 +490,27 @@ class Simple(object):
                 "Reduce Z was already sent today, try again tomorrow")
         self.till_closed = True
         self._save_state()
-        self.output.feed("REDUÇÃO Z\n")
-        self.output.feed_line()
+        self.write("REDUÇÃO Z\n")
+        self._feed_line()
 
     def till_add_cash(self, value):
-        self.output.feed('SUPRIMENTO: %s\n' % value)
-        self.output.feed_line()
+        self.write('SUPRIMENTO: %s\n' % value)
+        self._feed_line()
         self._check()
 
     def till_remove_cash(self, value):
-        self.output.feed('SANGRIA: %s\n' % value)
-        self.output.feed_line()
+        self.write('SANGRIA: %s\n' % value)
+        self._feed_line()
         self._check()
 
     def till_read_memory(self, start, end):
-        self.output.feed('LEITURA MF\n')
-        self.output.feed_line()
+        self.write('LEITURA MF\n')
+        self._feed_line()
         self._check()
 
     def till_read_memory_by_reductions(self, start, end):
-        self.output.feed('LEITURA MF\n')
-        self.output.feed_line()
+        self.write('LEITURA MF\n')
+        self._feed_line()
         self._check()
 
     def query_status(self):
@@ -476,7 +518,6 @@ class Simple(object):
         return None
 
     def get_serial(self):
-        self.output.show_all()
         self._check()
         return 'Virtual'
 
@@ -486,7 +527,7 @@ class Simple(object):
 
     def get_payment_constant(self, payment):
         self._check()
-        print 'payment', payment
+        print('payment', payment)
 
     def get_payment_constants(self):
         self._check()
@@ -513,23 +554,19 @@ class Simple(object):
         self._check()
         return self.ccf
 
-    def identify_customer_at_end(self):
-        self._check()
-        return False
-
     def get_sintegra(self):
         self._check()
-        return Settable(opening_date=self.opening_date,
-                        serial=self.serial,
-                        serial_id=self.serial_id,
-                        coupon_start=self.coupon_start,
-                        coupon_end=self.coupon_end,
-                        cro=self.cro,
-                        crz=self.crz,
-                        coo=self.coo,
-                        period_total=self.period_total,
-                        total=self.total,
-                        taxes=self.taxes)
+        return SintegraData(opening_date=self.opening_date,
+                            serial=self.serial,
+                            serial_id=self.serial_id,
+                            coupon_start=self.coupon_start,
+                            coupon_end=self.coupon_end,
+                            cro=self.cro,
+                            crz=self.crz,
+                            coo=self.coo,
+                            period_total=self.period_total,
+                            total=self.total,
+                            taxes=self.taxes)
 
     def get_crz(self):
         return self.crz
@@ -541,24 +578,27 @@ class Simple(object):
         return None
 
     def payment_receipt_open(self, identifier, coo, method, value):
-        self.output.feed("    RECIBO DE PAGAMENTO coo=%s\n" % coo)
+        self.write("    RECIBO DE PAGAMENTO coo=%s\n" % coo)
 
     def payment_receipt_print(self, text):
         self._check()
-        self.output.feed(text)
+        self.write(text)
 
     def payment_receipt_close(self):
         self._check()
-        self.output.feed_line()
+        self._feed_line()
 
     def gerencial_report_open(self, gerencial_id=0):
         self._check()
+        self.write('      RELATORIO GERENCIAL\n\n')
 
     def gerencial_report_print(self, text):
         self._check()
+        self.write(text)
 
     def gerencial_report_close(self):
         self._check()
+        self._feed_line()
 
     def get_firmware_version(self):
         return '1.1.1'
@@ -566,5 +606,53 @@ class Simple(object):
     #
     # IChequePrinter implementation
     #
+
     def print_cheque(self, value, thirdparty, city, date=None):
         return
+
+    #
+    # INonFiscalPrinter implementation
+    #
+
+    def centralize(self):
+        self._is_centralized = True
+
+    def descentralize(self):
+        self._is_centralized = False
+
+    def set_bold(self):
+        self._is_bold = True
+
+    def unset_bold(self):
+        self._is_bold = False
+
+    def set_condensed(self):
+        self._is_condensed = True
+
+    def unset_condensed(self):
+        self._is_condensed = False
+
+    def set_double_height(self):
+        self._is_double_height = True
+
+    def unset_double_height(self):
+        self._is_double_height = False
+
+    def print_line(self, data):
+        self._check()
+        if self._is_centralized:
+            data = data.center(self.max_characters)
+        self.write(data + b'\n')
+
+    def print_inline(self, data):
+        self._check()
+        self.write(data)
+
+    def print_barcode(self, code):
+        self.write('=== BARCODE {} ===\n'.format(code))
+
+    def print_qrcode(self, code):
+        self.write('=== QRCODE {} ===\n'.format(code))
+
+    def cut_paper(self):
+        self.write('\n--- paper cut ---\n')
